WITH Ada.Text_IO;
WITH Ada.Numerics.Discrete_Random;
WITH Screen;
PACKAGE BODY Spider IS
----------------------------------------------------------------------
--| This package provides procedures to emulate "Spider" 
--| commands. The spider can move around the screen drawing 
--| simple patterns. This version is for a 24 x 80 screen.
--| Original Author: John Dalbey, Cal Poly San Luis Obispo, 1992
--| Adapted by: Michael B. Feldman, The George Washington University
--| Last Modified: December 1998
----------------------------------------------------------------------

  -- Spider's View of her Room - rows and cols both numbered 1..20
  SUBTYPE Rows IS Positive RANGE 1..20;
  SUBTYPE Cols IS Positive RANGE 1..20;
  RowsInRoom   : CONSTANT Positive := Rows'Last;
  ColsInRoom   : CONSTANT Positive := Cols'Last;

  -- Screen Description Constants: for 24 x 80 screen, 
  -- 1 spider row = 1 screen row, 1 spider col = 2 screen cols
  RowLow       : CONSTANT Screen.Depth := 2;  -- room row bounds
  RowHigh      : CONSTANT Screen.Depth := RowLow + Rows'Last; 
  ColLow       : CONSTANT Screen.Width := 21; -- lower column bound

  -- Spider Current State Information
  Spidersym    : CONSTANT character := '*';   
  CurrentColumn: Cols;                  -- spider's position
  CurrentRow   : Rows;                  --   in the room.
  Heading      : Directions;            -- spider's direction
  Ink          : Colors;                -- spider's color   

  -- internal procedures and functions, not in specification
  -- and therefore not available to client program
  
  FUNCTION ColorSymbols (Color: Colors) RETURN Character IS
  -- Pre:  Color is defined
  -- Post: Returns the drawing character corresponding to Color
  BEGIN
    CASE Color IS
      WHEN Red   => RETURN 'R';
      WHEN Blue  => RETURN 'B';
      WHEN Green => RETURN 'G';
      WHEN Black => RETURN 'K';
      WHEN None  => RETURN '.';
    END CASE;
  END ColorSymbols;
  
  FUNCTION Compass (Direction: Directions) RETURN Character IS
  -- Pre:  Direction is defined
  -- Post: Returns drawing character corresponding to Direction
  BEGIN
    CASE Direction IS
      WHEN North => RETURN '^';
      WHEN East  => RETURN '>';
      WHEN South => RETURN 'v';
      WHEN West  => RETURN '<';
    END CASE;
  END Compass;

  PROCEDURE DrawSymbol (Which: Character) IS
  -- Pre:  Which is defined
  -- Post: Which appears in its proper position on the screen
  BEGIN
    Screen.MoveCursor (Row => (RowLow - 1) + CurrentRow,
                       Column => (ColLow - 2) + (2 * CurrentColumn));
    Ada.Text_IO.Put (Item => Which);
    Ada.Text_IO.Flush;
  END DrawSymbol;

  PROCEDURE DrawStatus IS
  -- Pre:  None
  -- Post: Status Box appears on the screen
  BEGIN
    Screen.MoveCursor (Row => 2, Column => 1);
    Ada.Text_IO.Put (" --- ");
    Screen.MoveCursor (Row => 3, Column => 1);
    Ada.Text_IO.Put ("|   |");
    Screen.MoveCursor (Row => 4, Column => 1);
    Ada.Text_IO.Put ("|   |");
    Screen.MoveCursor (Row => 5, Column => 1);
    Ada.Text_IO.Put (" --- ");
  END DrawStatus;

  PROCEDURE DrawRoom  IS
  -- Pre:  None
  -- Post: Room appears on the screen
  BEGIN
    Screen.ClearScreen;
    Screen.MoveCursor (Row => 1, Column => 1);
    -- Top Bar
    Ada.Text_IO.Put ("                   ");
    Ada.Text_IO.Put (" --------------------------------------- ");
    Ada.Text_IO.New_Line;
    FOR I in 1..20 LOOP
    Ada.Text_IO.Put ("                   ");
    Ada.Text_IO.Put ("|. . . . . . . . . . . . . . . . . . . .|");
    Ada.Text_IO.New_Line;
    END LOOP;
    Ada.Text_IO.Put ("                   ");
    Ada.Text_IO.Put (" --------------------------------------- ");
    DrawStatus;
  END DrawRoom;

  PROCEDURE ChangeColor (NewColor : Colors) IS
  -- Pre:  NewColor is defined
  -- Post: Ink is changed to NewColor and displayed in status box
  BEGIN
    Ink := NewColor;
    Screen.MoveCursor ( Row => 4, Column => 3);
    Ada.Text_IO.Put (ColorSymbols(Ink));
  END ChangeColor;

  PROCEDURE ShowDirection IS
  -- Pre:  None
  -- Post: Heading is displayed in the status box
  BEGIN
    Screen.MoveCursor(Row => 3,Column => 3);
    Ada.Text_IO.Put (Compass(Heading));
  END ShowDirection;

  PROCEDURE ShowSpider IS
  -- Pre:  None
  -- Post: The spider symbol appears in its current position
  BEGIN
    DrawSymbol (SpiderSym);
  END ShowSpider;

  -- Random number generators; instances of the generic one
  PACKAGE RandomSteps IS NEW Ada.Numerics.Discrete_Random
    (Result_Subtype => Steps);
  GSteps: RandomSteps.Generator;  

  PACKAGE RandomColors IS NEW Ada.Numerics.Discrete_Random
    (Result_Subtype => Colors);
  GColors : RandomColors.Generator;  

  PACKAGE RandomDirections IS NEW Ada.Numerics.Discrete_Random
    (Result_Subtype => Directions);
  GDirections : RandomDirections.Generator;  

  -- These procedures are in the package specification
  -- and implement the "official" spider commands
   
  PROCEDURE Start IS
  BEGIN
    DrawRoom;
    CurrentColumn := 10; -- these are in the spider's view
    CurrentRow := 11;
    Heading := North;
    ChangeColor(NewColor => Green);
    ShowSpider;
    ShowDirection;
    RandomSteps.Reset(Gen => GSteps);
    RandomColors.Reset(Gen => GColors);
    RandomDirections.Reset(Gen => GDirections);
  END Start;

  PROCEDURE TurnRight IS
  BEGIN
    IF Heading = Directions'Last THEN
      Heading := Directions'First;
    ELSE
      Heading := Directions'Succ (Heading);
    END IF;
    ShowDirection;
  END TurnRight; 

  PROCEDURE Face (WhichWay: IN Directions) IS
  BEGIN
    Heading := WhichWay;
    ShowDirection;
  END Face;  

  FUNCTION IsPainting RETURN Colors IS   
  BEGIN
    RETURN Ink;
  END IsPainting;

  FUNCTION IsFacing RETURN Directions IS   
  BEGIN
    RETURN Heading;
  END IsFacing;

  FUNCTION AtWall RETURN Boolean IS
  BEGIN
    -- Check for out of bounds (in the spider's view)
    CASE Heading IS
      WHEN North => 
        RETURN CurrentRow <= Rows'First;
      WHEN East  => 
        RETURN CurrentColumn >= Cols'Last;
      WHEN South => 
        RETURN CurrentRow >= Rows'Last;
      WHEN West  => 
        RETURN CurrentColumn <= Cols'First;
    END CASE;
  END AtWall;

  FUNCTION RandomStep RETURN Steps IS
  BEGIN
    RETURN RandomSteps.Random(GSteps);
  END RandomStep;

  FUNCTION RandomColor RETURN Colors IS
  BEGIN
    RETURN RandomColors.Random(GColors);
  END RandomColor;

  FUNCTION RandomDirection RETURN Directions IS
  BEGIN
    RETURN RandomDirections.Random(GDirections);
  END RandomDirection;

  PROCEDURE Step IS
  BEGIN

    -- leave a track where spider is standing
    DrawSymbol (ColorSymbols (Ink) );
    
    -- If out of bounds raise exception.
    IF AtWall THEN
      Screen.Beep;
      RAISE Hit_the_Wall;
    END IF;

    -- change the spider's location 
    CASE Heading IS
      WHEN North =>     
        CurrentRow := CurrentRow - 1;
      WHEN East  => 
        CurrentColumn := CurrentColumn + 1;
      WHEN South => 
        CurrentRow := CurrentRow + 1;
      WHEN West  => 
        CurrentColumn := CurrentColumn - 1;
    END CASE;

    -- draw the spider in her new location
    DrawSymbol (SpiderSym);

    -- if debug mode, wait for user to press RETURN
    IF Debugging = On THEN   
      Ada.Text_IO.Skip_Line;
    ELSE
      DELAY 0.2;
    END IF;
  END Step;
  
  PROCEDURE Quit IS
  -- Quit command.
  BEGIN
    Screen.MoveCursor(Row => 23,Column => 1);
  END Quit;

  DebugFlag    : Boolean := False;      -- Is single stepping on?

  PROCEDURE Debug (Setting: Switch) is
  -- Toggle debugging mode
  BEGIN
    IF Setting = ON THEN
      DebugFlag := true;
      Screen.MoveCursor (Row => 10,Column => 1);
      Ada.Text_IO.Put ("-- DEBUG ON -- ");
      Ada.Text_IO.New_Line;
      Ada.Text_IO.Put ("  Press Enter");
    ELSE
      DebugFlag := false;
      Screen.MoveCursor (Row => 10,Column => 1);
      Ada.Text_IO.Put ("               ");
      Ada.Text_IO.New_Line;
      Ada.Text_IO.Put ("             ");
    END IF;
  END Debug;

  FUNCTION Debugging RETURN Switch IS 
  BEGIN
    IF DebugFlag THEN
      RETURN On;
    ELSE
      RETURN Off;
    END IF;
  END Debugging;

END Spider;
